package icse2011experiments;

import java.io.PrintWriter;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.util.filter.IsChangeOperationFilter;
import ca.mcgill.cs.swevo.util.CollectionUtil;
  
public class DiffTypeCounter {

	private final String[] CHANGE_TYPES = new String[] {
		ChangeOperation.AFFECTED_ONLY_BY_RENAMES.name(),
		ChangeOperation.ATTRIBUTE_NAME_UPDATE.name(),
		ChangeOperation.ATTRIBUTE_TYPE_TRIVIAL_UPDATE.name(),
		ChangeOperation.ATTRIBUTE_TYPE_RENAME_UPDATE.name(),
		ChangeOperation.CLASS_NAME_UPDATE.name(),
		ChangeOperation.LOCAL_VARIABLE_EXTRACTION_INSERT.name(),
		ChangeOperation.METHOD_NAME_UPDATE.name(),
		ChangeOperation.PARAMETER_NAME_UPDATE.name(),
		ChangeOperation.PARAMETER_TYPE_RENAME_UPDATE.name(),
		ChangeOperation.PARAMETER_TYPE_TRIVIAL_UPDATE.name(),
		ChangeOperation.RETURN_TYPE_RENAME_UPDATE.name(),
		ChangeOperation.RETURN_TYPE_TRIVIAL_UPDATE.name(),
		ChangeOperation.THROWN_EXCEPTION_RENAME_UPDATE.name(),
		ChangeOperation.THROWN_EXCEPTION_TRIVIAL_UPDATE.name(),
		ChangeOperation.TRIVIAL_METHOD_INVOCATION_DELETE.name(),
		ChangeOperation.TRIVIAL_METHOD_INVOCATION_INSERT.name(),
		ChangeOperation.TRIVIAL_RETURN_STATEMENT_DELETE.name(),
		ChangeOperation.TRIVIAL_THIS_EXPRESSION_INSERT.name(),
		ChangeOperation.VARIABLE_NAME_UPDATE.name(),
		ChangeOperation.VARIABLE_DECLARED_TYPE_RENAME_UPDATE.name(),
		ChangeOperation.VARIABLE_DECLARED_TYPE_TRIVIAL_UPDATE.name()};
	
	private final List<IsChangeOperationFilter> filters; {
		filters = new LinkedList<IsChangeOperationFilter>();
		for (String changeType : CHANGE_TYPES) {
			filters.add(new IsChangeOperationFilter(changeType));
		}
	}
	
	public void record (PrintWriter pw) {
		pw.print("id");
		for (IsChangeOperationFilter filter : filters) {
			pw.print(",");
			pw.print(CollectionUtil.getFirst(filter.getOps()));
		}
		pw.println();
	}
	
	public void record (int id, Collection<DiffCatResult> results, PrintWriter pw) {
		pw.print(id);
		for (IsChangeOperationFilter filter : filters) {
			Collection<DiffCatResult> retained = CollectionUtil.filterAndCopy(results, filter);
			pw.print(",");
			pw.print(retained.size());
		}
		pw.println();
	}
}
