package icse2011experiments;

import java.io.File;
import java.io.PrintWriter;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.IMethodBinding;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.MethodDeclaration;

import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.model.DiffCodes;
import ca.mcgill.cs.swevo.diffcat.util.DiffCatParentMap;
import ca.mcgill.cs.swevo.util.CollectionUtil;
import ca.mcgill.cs.swevo.util.LRTuple;
import ca.mcgill.cs.swevo.util.LRTupleSet;
 
public class ItemSetRecorder {

	private static final String ITEM_SEP = ",";
	private static final String PARAM_SEP = "#";
	private static final String REFACTOR  = "R";
	private PrintWriter pw;
	
	public ItemSetRecorder (String file) {
		try {
			pw = new PrintWriter(new File(file));
		} catch (Exception e) {
			
		}
	}
	
	public void recordAll (int id, Collection<DiffCatResult> allDiffs) {
		
		// write down refactorings
		Collection<DiffCatResult> sigRefactorings = CollectionUtil.filterAndCopy(allDiffs, DiffCodes.METHOD_SIG_REFACTOR_DIFF); 
		
		LRTupleSet<IMethodBinding> toRefactor = new LRTupleSet<IMethodBinding>();
		
		DiffCatParentMap map = new DiffCatParentMap(sigRefactorings);
		for (LRTuple<ASTNode> methodPair : map.getMethodKeys()) {
			MethodDeclaration leftDecl = (MethodDeclaration) methodPair.getLeft();
			MethodDeclaration rightDecl = (MethodDeclaration) methodPair.getRight();
			
			toRefactor.add(new LRTuple<IMethodBinding>(leftDecl.resolveBinding(), rightDecl.resolveBinding())); 
		}
		
		doRefactorings(toRefactor);
		
		// collect bindings
		Collection<DiffCatResult> methodUpdates = CollectionUtil.filterAndCopy(allDiffs, DiffCodes.METHOD_BODY_DIFF);
		
		map = new DiffCatParentMap(methodUpdates);
		Set<IMethodBinding> touched = new HashSet<IMethodBinding>();
		for (LRTuple<ASTNode> methodPair : map.getMethodKeys()) {
			MethodDeclaration rightDecl = (MethodDeclaration) methodPair.getRight();
			touched.add(rightDecl.resolveBinding());
		}
		
		// print item set
		pw.print(id);
		pw.print(ITEM_SEP);
		printBindings(pw, touched);
		
		// flush
		pw.println();
		pw.flush();
	}
	
	
	private void printBindings (PrintWriter pw, Collection<IMethodBinding> bindings)
	{
		Set<String> converted = new HashSet<String>();
		for (IMethodBinding binding : bindings){
			if (binding == null) {
				continue;
			}
			converted.add(getString(binding));
		}
		
		String[] bindingArr = converted.toArray(new String[converted.size()]);
		if (bindingArr.length == 0) {
			return;
		}
		pw.print(bindingArr[0]);
		for (int i = 1; i < bindingArr.length; i++) {
			pw.print(ITEM_SEP);
			pw.print(bindingArr[i]);
		}
	}
	
	private String getString (IMethodBinding binding) 
	{
		StringBuffer buf = new StringBuffer();
		buf.append(getString(binding.getDeclaringClass()));
		buf.append(".");
		buf.append(binding.getName());
		buf.append("(");
		ITypeBinding[] params = binding.getParameterTypes();
		if (params.length > 0) {
			buf.append(getString(params[0]));
			for (int i = 1; i < params.length; i++) {
				buf.append(PARAM_SEP);
				buf.append(getString(params[i]));
			}
		}
		buf.append(")");
		return buf.toString();
	}
	
	private String getString (ITypeBinding binding) 
	{
		return binding.getQualifiedName();
	}
	
	private void doRefactorings (LRTupleSet<IMethodBinding> refactorings) {
		
		for (LRTuple<IMethodBinding> refactoring : refactorings) {
			pw.print(REFACTOR);
			pw.print(ITEM_SEP);
			pw.print(getString(refactoring.getLeft()));
			pw.print(ITEM_SEP);
			pw.print(getString(refactoring.getRight()));
			pw.println();
		}
	}
	
	
}
