package icse2011experiments.assoc;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.List;

/**
 * Helper program to standardize refactorings detected in change history. 
 * Takes last known refactored version of a given method signature and and
 * makes sure this one is used to do associations (instead of losing associations
 * whenever a method or class is refactored)
 * 
 * @author dkawry
 */
public class RefactoringStandardizer {

	public static void main(String[] args) {
		
		// change this to reflect the actual home path
		final String HOME = "/home/2005/dkawry/research/experiments/icse11";
		
		// change this to reflect the project analyzed
		final String PROJECT = "xerces";
		
		// change these to [ant, hibernate, jdt_core, jdt_ui, spring, vuze, xerces] to do different files 
		String readFromFile = HOME + "/weka/"+ PROJECT + "/ess3.txt";
		String printToFile = HOME + "/weka/" + PROJECT + "/essR.txt";
		
		// get the lines 
		List<String> rawStrings = Util.readLines(readFromFile);
		
		// go backwards and apply refactorings to previous methods
		for (int i = rawStrings.size() - 1; i > -1; i--) {
			String line = rawStrings.get(i);
			String[] elements = line.split(",");
			
			// refactorings are always R,old,new
			if (elements.length != 3 || ! elements[0].equals("R")) {
				continue;
			}
			
			String oldM = elements[1];
			String newM = elements[2];
			
			// replace all occurrences of oldM with newM 
			for (int j = i - 1; j > -1; j--) {
				String line2 = rawStrings.get(j);
				String[] elements2 = line2.split(",");
				for (int z = 0; z < elements2.length; z++) {
					if (elements2[z].equals(oldM)) {
						elements2[z] = newM;
					}
				}
				StringBuffer buf = new StringBuffer();
				if (elements2.length >= 1) {
					buf.append(elements2[0]);
				}
				for (int z = 1; z < elements2.length; z++) {
					buf.append(",");
					buf.append(elements2[z]);
				}
				rawStrings.set(j, buf.toString());
			}
		}
		
		// print the updated file
		PrintWriter pw = null;
		try {
			pw = new PrintWriter(new File(printToFile));
		} catch (FileNotFoundException e) {
			throw new RuntimeException();
		}
		
		for (int i = 0; i < rawStrings.size(); i++) {
			// don't bother writing refactorings anymore
			if (rawStrings.get(i).startsWith("R,")) {
				continue;
			}
			pw.println(rawStrings.get(i));
		}
		
		pw.close();
	}
}
